/*! babel-preset-edge v4.0.1 by Sebastian Software <s.werner@sebastian-software.de> */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _objectSpread = _interopDefault(require('@babel/runtime/helpers/builtin/objectSpread'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/builtin/typeof'));
var appRootDir = require('app-root-dir');
var semver = _interopDefault(require('semver'));
var browserslist = _interopDefault(require('browserslist'));
var getTargets = _interopDefault(require('@babel/preset-env/lib/targets-parser'));
var fastAsyncPlugin = _interopDefault(require('fast-async'));
var envPreset = require('@babel/preset-env');
var envPreset__default = _interopDefault(envPreset);
var envPlugins = _interopDefault(require('@babel/preset-env/data/plugins.json'));
var minifyPreset = _interopDefault(require('babel-preset-minify'));
var path = require('path');
var moduleResolver = _interopDefault(require('babel-plugin-module-resolver'));
var lodashPlugin = _interopDefault(require('babel-plugin-lodash'));
var transformRuntimePlugin = _interopDefault(require('@babel/plugin-transform-runtime'));
var dynamicImportSyntaxPlugin = _interopDefault(require('babel-plugin-syntax-dynamic-import'));
var dynamicImportRollupNode = _interopDefault(require('babel-plugin-dynamic-import-node'));
var dynamicImportServerRendering = _interopDefault(require('loadable-components/babel'));
var dynamicImportSmartWebpack = _interopDefault(require('babel-plugin-smart-webpack-import'));
var classPropertiesPlugin = _interopDefault(require('@babel/plugin-proposal-class-properties'));
var objectRestSpreadPlugin = _interopDefault(require('@babel/plugin-proposal-object-rest-spread'));
var reactPreset = _interopDefault(require('@babel/preset-react'));
var flowPreset = _interopDefault(require('@babel/preset-flow'));
var reactInlineElementsPlugin = _interopDefault(require('@babel/plugin-transform-react-inline-elements'));
var reactConstantElements = _interopDefault(require('@babel/plugin-transform-react-constant-elements'));
var reactIntlPlugin = _interopDefault(require('babel-plugin-react-intl'));
var transformRemovePropTypes = _interopDefault(require('babel-plugin-transform-react-remove-prop-types'));

/* eslint-disable max-len */
var defaults = {
  // Whether to print hints on transpilation settings which were selected.
  debug: false,
  // One of the following:
  // - "node": Targetting NodeJS (Outputs CommonJS modules, translates dynamic imports into require statements, ...)
  // - "browser": Targetting Browsers (Expects Webpack, Parcel or Rollup post-processing, keeps dynamic import, ...)
  // - "universal": Targetting both NodeJS and Browsers (Ideally suited for libraries published for both NodeJS and browsers via NPM)
  // - "auto": Define target automatically based on NODE_ENV environment.
  target: "auto",
  // Choose which transpilation should be applied.
  // One of the following:
  // - "es5": Standard output for widest engine and browser support. Ideally suited for publishing to NPM.
  // - "es2015": Alternative to standard es5 reaching only modern engines and browsers which support at least all features of es2015. Might be a good alternative for publishing modern libraries to NPM or when using transpilation on all content - even `node_modules` in application code.
  // - "modern": Uses a built-in definition of modern NodeJS and browser versions. This is interesting for local development of application as it accelerates features like hot-loading quite a bit.
  // - "current": NodeJS only. Ideally for local running test suites, etc. Using the least amount of transpile for making code runnable locally.
  // - "node": NodeJS only. Uses `engines` field in `package.json` to define the NodeJS version to target.
  // - "browser": Browser only. Uses local "browserslist" config to determine transpilation target. Uses `BROWSERSLIST_ENV` if configured. Otherwise uses `env` passed through preset or via `NODE_ENV`.
  // - "auto": Uses "browser" for `target: browser`. Uses "node" for `target: node`. Uses `es5` for `target: universal`.
  // - {}: A custom object which is passed to `@babel/preset-env`
  transpile: "auto",
  // Select environment where we are in for the current job.
  // One the the following:
  // - "development": During development of applications, publishing of libraries containing debug code
  // - "production": Publishing application to the public, publishing of clean non-debug code containing libraries and command line applications.
  // - "test": Used for testing e.g. with Ava or Jest test runners.
  // - "auto": Automatically using the `EDGE_ENV` or `NODE_ENV` environment variables.
  env: "auto",
  // Choose whether and how imports should be processed.
  // - "cjs": Transpile module imports to CommonJS
  // - false: Keep module imports
  // - "auto": Automatic selection based on `target`.
  modules: "auto",
  // Choose automatically depending on target by default or use one of these for full control:
  // - "node": For usage in NodeJS (e.g. produce binaries), publish NodeJS-only libraries.
  // - false: Enhance imports with further details (chunk names, SSR support), but keep them functional as they are.
  imports: false,
  // Replace the function used when compiling JSX expressions. Default: React.
  // See also: https://www.npmjs.com/package/@babel/preset-react
  jsxPragma: null,
  // Replace the component used when compiling JSX fragments. Default: React.
  // See also: https://www.npmjs.com/package/@babel/preset-react
  jsxPragmaFrag: null,
  // Transpilation Settings: We default on a loose transpilation which is efficient
  // but not overly compliant. If you experience issues it might be better to
  // switch `looseMode` off. `specMode` on the other hand might produce
  // 100% correct code, but tend to be large and slower as well.
  looseMode: true,
  specMode: false,
  // Lodash Plugin Settings. Optimizes import statements for smaller bundles.
  // The idea behind here is that some libraries are publishing individual functions into individual files.
  // This helps tree-shaking until all libraries are correctly dealing with side-effect flags and bundlers have better support.
  optimizeModules: ["lodash", "async", "rambda", "recompose"],
  // Configuration for module lookup
  sourceFolder: "src",
  // Whether to enable source map output
  sourceMaps: true,
  // Enable full compression on production scripts or basic compression for libraries or during development.
  compression: true,
  // Whether to apply more agressive minification. Automatically enabled when using `compression: true` and running in production env.
  minified: null
};

function isProduction(options) {
  return /\bproduction\b/.test(options.env) && !isDevelopment(options);
}
function isDevelopment(options) {
  return /\bdevelopment\b/.test(options.env) && /\btest\b/.test(options.env);
}

/* eslint-disable complexity, immutable/no-mutation */

function getOptions() {
  var input = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  // These are the final options we use later on.
  var output = _objectSpread({}, defaults, input); // Normalize "auto"/null. Makes processing later in code better.


  if (output.target == null) {
    output.target = "auto";
  }

  if (output.transpile == null) {
    output.transpile = "auto";
  }

  if (output.env == null) {
    output.env = "auto";
  }

  if (output.minified == null) {
    output.minified = "auto";
  }

  if (output.modules == null) {
    output.modules = "auto";
  } // Fallback to environment variables when possible.


  if (output.env == null) {
    output.env = process.env.EDGE_ENV || process.env.BABEL_ENV || process.env.NODE_ENV || "development";
  }

  if (output.target === "auto") {
    if (/\btest\b/.test(output.env)) {
      output.target = "node";
    } else if (/\bnode\b/.test(output.env)) {
      output.target = "node";
    } else if (/\bbrowser\b/.test(output.env)) {
      output.target = "browser";
    } else {
      output.target = "universal";
    }
  }

  if (output.debug) {
    console.log("- Target:", output.target);
  } // Auto select test target when running in test environment


  if (output.transpile === "auto") {
    if (/\btest\b/.test(output.env)) {
      output.transpile = "current";

      if (output.debug) {
        console.log("- Selecting `transpile: current` based on environment.");
      }
    } else if (output.target === "browser") {
      output.transpile = "browser";
    } else if (output.target === "node") {
      output.transpile = "node";
    } else if (/\bmodern\b/.test(output.env)) {
      output.transpile = "modern";

      if (output.debug) {
        console.log("- Selected 'modern' transpile based on ENV");
      }
    } else if (/\bes2015\b/.test(output.env)) {
      output.transpile = "es2015";

      if (output.debug) {
        console.log("- Selected 'es2015' transpile based on ENV");
      }
    } else {
      output.transpile = "es5";
    }
  }

  if (output.debug) {
    console.log("- Transpile:", output.transpile);
  } // Automatic detection of "modules" mode based on target


  if (output.modules === "auto") {
    if (output.target === "node") {
      output.modules = "cjs";
    } else {
      // Libraries should be published as EcmaScript modules for tree shaking support
      // For browser targets we typically use tools like Webpack, Rollup or Parcel which benefit from EcmaScript modules, too.
      output.modules = false;
    }
  }

  if (output.minified === "auto") {
    output.minified = output.compression && isProduction(output);
  }

  if (output.debug) {
    console.log("Final Options:", output);
  }

  return output;
}

function shouldPrintComment(comment) {
  // Keep pure function markers which are generated by some plugins
  // See sideEffects option: https://github.com/mishoo/UglifyJS2
  if (/[#@]__PURE__/.exec(comment)) {
    return true;
  } // Keep JSON5 magic comments used for Webpack hints


  if (/^\s?webpack[A-Z][A-Za-z]+:/.exec(comment)) {
    return true;
  }

  return false;
}

var modernTarget = {
  node: "8.9.0",
  electron: "1.8",
  browsers: ["Safari >= 11.1", "iOS >= 11.3", "Edge >= 16", "Chrome >= 64", "ChromeAndroid >= 64", "Firefox >= 58"]
};

var engines = require("".concat(appRootDir.get(), "/package.json")).engines;
/* eslint-disable immutable/no-mutation, complexity */


function getEnvTargets(options) {
  var envTargets = {}; // if (options.target === "node") {
  //   // Widely used stable NodeJS (LTS) is v6.9.0
  //   // See also: https://nodejs.org/en/blog/release/v6.9.0/
  //   // Newest LTS is v8.9.0 https://nodejs.org/en/blog/release/v8.9.0/
  //   // You can choose the modern version by setting `target` to "node8".
  //   // We also have support for even more modern Node v10 which did not yet reached LTS.
  //   envTargets.node =
  //     options.target === "node8" ?
  //       "8.9.0" :
  //       options.target === "node10" ?
  //         "10.0.0" :
  //         "6.9.0"
  //   envTargets.browsers = []
  // }

  if (_typeof(options.transpile) === "object") {
    envTargets = options.transpile;
  } else if (options.transpile === "current") {
    // Scripts which are directly used like tests can be transpiled for the current NodeJS version
    envTargets.node = "current";
    envTargets.browsers = [];
  } else if (options.transpile === "browser") {
    // What we do here is actually pretty clever/stupid as we just use browserslist
    // itself to query its configuration and pass over that data again to babel-preset-env
    // for passing it to browserslist internally. Yeah.
    envTargets.browsers = browserslist(null, {
      env: process.env.BROWSERSLIST_ENV || isProduction(options) ? "production" : "development"
    });
  } else if (options.transpile === "modern") {
    envTargets = modernTarget;
  } else if (options.transpile === "es5" || options.transpile === "es2015") {
    // Compilation with "latest" preset supporting a wide range of clients via ES5 output
    // For ignoring any existing browserslist config we have to pass over an empty array.
    envTargets.browsers = [];
  } else if (options.transpile === "node") {
    // Using NodeJS version from engine field
    if (engines && engines.node) {
      // LTS Overview as of May 2018
      // 4.2.0 - out of support
      // 6.9.0 - maintenance
      // 8.9.0 - active
      if (semver.satisfies("4.2.0", engines.node)) {
        envTargets.node = "4.2.0";
      } else if (semver.satisfies("6.9.0", engines.node)) {
        envTargets.node = "6.9.0";
      } else if (semver.satisfies("8.9.0", engines.node)) {
        envTargets.node = "8.9.0";
      } else {
        throw new Error("Invalid transpile configuration! Unable to match required engines.");
      }
    } else {
      // As Node v4 is out of support, we use the current LTS as default
      // See also: https://github.com/nodejs/Release
      // This behavior reduces the amount a transpilations a bit when working in pure NodeJS environments
      envTargets.node = "6.9.0";
    }
  } else {
    throw new Error("Invalid transpile configuration!");
  } // We fill the browsers field if not defined with an empty array so that
  // the browserslist config is properly ignored when transpilation if passed over.


  if (envTargets.node != null && envTargets.browsers == null) {
    envTargets.browsers = [];
  }

  if (options.debug) {
    console.log("- Computed Targets:", envTargets);
    console.log("- Final Targets:", getTargets(envTargets));
  }

  return envTargets;
}

function async(presets, plugins, options) {
  /* eslint-disable immutable/no-mutation */
  // Directly ask babel-preset-env whether we want to use transform-async
  // based on currently configured targets. Only if that's the case we
  // transform our async/await code. Otherwise we assume it works without
  // any transpilation.
  var requiresAsync = envPreset.isPluginRequired(getTargets(getEnvTargets(options)), envPlugins["transform-async-to-generator"]); // Alternative to Babel Regenerator
  // Implements the ES7 keywords async and await using syntax transformation
  // to at Promises at compile-time, rather than using generators.
  // https://github.com/babel/babel/pull/7076 (NEW: bundled plugin with Babel)
  // https://www.npmjs.com/package/fast-async (OLD: separate Babel plugin)

  if (requiresAsync) {
    plugins.push(fastAsyncPlugin);
  }

  if (options.debug) {
    console.log("- Async/Await Transpilation:", requiresAsync);
  }
}

var allDisabled = {
  booleans: false,
  builtIns: false,
  consecutiveAdds: false,
  deadcode: false,
  evaluate: false,
  flipComparisons: false,
  guards: false,
  infinity: false,
  mangle: false,
  memberExpressions: false,
  mergeVars: false,
  numericLiterals: false,
  propertyLiterals: false,
  regexpConstructors: false,
  removeUndefined: false,
  replace: false,
  simplify: false,
  simplifyComparisons: false,
  typeConstructors: false,
  undefinedToVoid: false
};
var mediumCompression = {
  // Based on a black list of too agressive plugins/settings
  // Settings
  keepFnName: true,
  keepClassName: true,
  // Disable Agressive Plugins
  booleans: false,
  infinity: false,
  mangle: false,
  flipComparisons: false,
  simplify: false
};

var lowCompression = _objectSpread({}, allDisabled, {
  // Settings
  keepFnName: true,
  keepClassName: true,
  // Enable only very basic plugins
  deadcode: true,
  guards: true,
  memberExpressions: true,
  propertyLiterals: true,
  regexpConstructors: true,
  removeUndefined: true,
  simplifyComparisons: true,
  typeConstructors: true
});

function compression(presets, plugins, options) {
  // Use basic compression for development/bundling and full compression for production output.
  if (options.compression) {
    if (isProduction(options)) {
      presets.push(minifyPreset);

      if (options.debug) {
        console.log("- Output: Full Minification");
      }
    } else {
      presets.push([minifyPreset, mediumCompression]);

      if (options.debug) {
        console.log("- Output: Medium Compression");
      }
    }
  } // Even when no compression is enabled, it makes sense to remove
  // and code paths which are clearly never executed.
  else {
      presets.push([minifyPreset, lowCompression]);

      if (options.debug) {
        console.log("- Output: Low Compression");
      }
    }
}

function convenience(presets, plugins, options) {
  // Optimization for cheery-picking from lodash, asyncjs, ramba and recompose.
  // Auto cherry-picking es2015 imports from path imports.
  // https://www.npmjs.com/package/babel-plugin-lodash
  // https://github.com/acdlite/recompose#using-babel-lodash-plugin
  plugins.push([lodashPlugin, {
    id: options.optimizeModules
  }]); // Supports loading files in source folder without relative folders
  // https://github.com/tleunen/babel-plugin-module-resolver

  if (options.sourceFolder != null) {
    plugins.push([moduleResolver, {
      alias: {
        "~": path.resolve(appRootDir.get(), options.sourceFolder)
      }
    }]);
  }
}

function env(presets, plugins, options) {
  var exclude = [];
  exclude.push("transform-regenerator", "transform-async-to-generator"); // Exclude all es2015 features which are supported by the default es2015 babel preset.
  // This targets all es2015-capable browsers and engines.

  if (options.transpile === "es2015") {
    if (options.debug) {
      console.log("- Targetting ES2015 engines");
    }

    exclude.push("transform-template-literals", "transform-literals", "transform-function-name", "transform-arrow-functions", "transform-block-scoped-functions", "transform-classes", "transform-object-super", "transform-shorthand-properties", "transform-duplicate-keys", "transform-computed-properties", "transform-for-of", "transform-sticky-regex", "transform-unicode-regex", "transform-spread", "transform-parameters", "transform-destructuring", "transform-block-scoping", "transform-typeof-symbol");
  }

  presets.push([envPreset__default, {
    // Setting this to false will not transform modules.
    modules: options.modules,
    // Prefer built-ins which also prefers global polyfills which is the right thing to do
    // for most scenarios like SPAs and NodeJS environments.
    useBuiltIns: "usage",
    // Options to tweak the details of the implementation. If both are `false` the environment
    // preset is executed in default mode.
    loose: options.looseMode,
    spec: options.specMode,
    // Debug output of features, plugins and presets which are enabled.
    debug: false,
    // We prefer the transpilation of the "fast-async" plugin over the
    // slower and more complex Babel internal implementation.
    exclude: exclude,
    // Differ between development and production for our scope.
    // NodeJS is generally fine in development to match the runtime version which is currently installed.
    targets: getEnvTargets(options)
  }]); // Use helpers, but not polyfills, in a way that omits duplication.
  // For polyfills better use polyfill.io or another more sophisticated solution.

  plugins.push([transformRuntimePlugin, {
    helpers: true,
    regenerator: false,
    polyfill: false,
    useBuiltIns: true,
    useESModules: options.modules === false
  }]);

  if (options.debug) {
    /* eslint-disable no-nested-ternary */
    console.log("- Module Settings:", options.modules === false ? "ESM" : options.modules);
    console.log("- Transpilation Compliance:", options.specMode ? "SPEC" : options.looseMode ? "LOOSE" : "DEFAULT");
  }
}

function imports(presets, plugins, options) {
  // Support for new @import() syntax
  plugins.push(dynamicImportSyntaxPlugin); // Support for enhanced imported components with SSR support

  plugins.push(dynamicImportServerRendering); // Automatically add chunk names to imports() (for Webpack usage)

  plugins.push(dynamicImportSmartWebpack); // Transpile the parsed import() syntax for compatibility or extended features.

  if (options.imports === "node") {
    if (options.debug) {
      console.log("- Rewriting import() for NodeJS.");
    } // Compiles import() to a deferred require() for NodeJS


    plugins.push(dynamicImportRollupNode);
  } else {
    if (options.debug) {
      console.log("- Keeping import() statement as is.");
    }
  }
}

function proposals(presets, plugins, options) {
  // Support for ES7 Class Properties (currently stage-2)
  // class { handleClick = () => { } }
  plugins.push(classPropertiesPlugin); // Support for Object Rest Spread `...` operator in objects.
  // { ...todo, completed: true }

  plugins.push([objectRestSpreadPlugin, {
    useBuiltIns: true
  }]);
}

function react(presets, plugins, options) {
  // Support Flow Syntax
  presets.push(flowPreset); // Transform JSX and prefer built-in methods

  presets.push([reactPreset, {
    useBuiltIns: true,
    pragma: options.jsxPragma,
    pragmaFrag: options.jsxPragmaFrag,
    development: isDevelopment(options)
  }]);

  if (isProduction(options)) {
    // Remove unnecessary React propTypes from the production build.
    // https://github.com/oliviertassinari/babel-plugin-transform-react-remove-prop-types
    plugins.push([transformRemovePropTypes, {
      mode: "remove",
      removeImport: true
    }]); // Cleanup descriptions for translations from compilation output

    plugins.push(reactIntlPlugin); // Replaces the React.createElement function with one that is
    // more optimized for production.
    // NOTE: Symbol needs to be polyfilled.
    // https://babeljs.io/docs/plugins/transform-react-inline-elements/

    plugins.push(reactInlineElementsPlugin); // Hoists element creation to the top level for subtrees that
    // are fully static, which reduces call to React.createElement
    // and the resulting allocations. More importantly, it tells
    // React that the subtree hasn’t changed so React can completely
    // skip it when reconciling.
    // https://babeljs.io/docs/plugins/transform-react-constant-elements/

    plugins.push(reactConstantElements);
  }
}

/* eslint-disable filenames/match-exported, no-console, complexity */
function buildPreset(context, input) {
  var options = getOptions(input);
  var presets = [];
  var plugins = []; // Plugin/Preset Order is important in Babel as the AST is only processed in one step.
  // If two transforms both visit the "Program" node, the transforms will run in either plugin or preset order.
  //
  // - Plugins run before Presets.
  // - Plugin ordering is first to last.
  // - Preset ordering is last to first(!).
  //
  // Official Docs: https://new.babeljs.io/docs/en/plugins.html#plugin-preset-ordering
  // See also: https://jamie.build/babel-plugin-ordering.html
  // Mainly plugins

  proposals(presets, plugins, options);
  async(presets, plugins, options);
  convenience(presets, plugins, options);
  imports(presets, plugins, options); // Mainly presets
  // env => react => compression

  compression(presets, plugins, options);
  react(presets, plugins, options);
  env(presets, plugins, options); // Assemble final config

  return {
    // Babel basic configuration
    compact: options.minified || "auto",
    minified: options.minified,
    // Whether to enable source maps
    sourceMaps: options.sourceMaps,
    // Fine tune comment output
    shouldPrintComment: shouldPrintComment,
    // Add lists of presets and plugins
    presets: presets,
    plugins: plugins
  };
}

module.exports = buildPreset;
//# sourceMappingURL=index.cjs.js.map
